from setuptools import setup, find_packages

setup(
    name='sthg_fastapi_logs',
    version='0.1.10',
    packages=find_packages(),
    description='Python FastApi logs',
    long_description=open('sthg_fastapi_logs/README.md').read(),
    long_description_content_type='text/markdown',
    author='DongQing',
    author_email='maoyouyu@163.com',
    url='https://github.com/yourusername/your_package_name',
    install_requires=[
        # 依赖项列表
    ],
    classifiers=[
        # 包分类列表，例如：
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
    ],
)


import time
import functools
import traceback
import json
import uuid

from starlette.responses import StreamingResponse
from starlette.concurrency import iterate_in_threadpool
from .enumerate import CodeEnum
from .log_util import local_trace, server_logger, error_logger, access_logger

TOKEN_URL = 'http://192.168.1.243:9103/api/user/userInfoByToken'


def get_process_time(start_time, end_time):
    return '{}ms'.format(round((end_time - start_time) * 1000, 6))


def get_status(response):
    if hasattr(response, 'status_code'):
        print(123123, response.status_code)
        return identify_code(response.status_code)
    return CodeEnum.SUCCESS


def get_trace_id():
    if hasattr(local_trace, 'trace_id'):
        return local_trace.trace_id
    return ''


def get_request_method(request):
    return str(request.url)


def get_ip(request):
    return request.client.host


def get_header(request):
    return dict(request.headers)


async def get_request_params(request):
    print("request", request)
    params = dict(request.query_params) if request.query_params else "-"
    if not params:
        byte_body = await request.body()
        params = json.loads(byte_body.decode()) if byte_body else "-"
    return params


async def get_response(response):
    # 如果响应是一个 StreamingResponse，我们需要特殊处理
    if isinstance(response, StreamingResponse):
        # 我们需要创建一个新的 StreamingResponse
        # 并且修改它的响应体
        async def new_streaming_response(stream):
            async for chunk in stream:
                # 在这里可以处理每个 chunk，例如记录日志、修改内容等
                print(chunk)  # 打印或者处理 chunk
                yield chunk  # 发送 chunk

        return StreamingResponse(new_streaming_response(response.body_iterator), media_type=response.media_type)
    else:
        # 对于非 StreamingResponse，可以直接修改 response.body
        data = await response.body()
        modified_data = data  # 在这里可以修改数据
        return Response(modified_data, media_type=response.media_type, status_code=response.status_code)
    # return data


def identify_code(code):
    code = int(code)
    ranges = {
        CodeEnum.SUCCESS.value: (199, 207),
        CodeEnum.REEOR.value: (1, 1),
        CodeEnum.PARAM_ERROR.value: (400, 499),
        CodeEnum.INNER_REEOR.value: (500, 10000)
        # 可以添加更多的范围
    }

    for identifier, (start, end) in ranges.items():
        if start <= code <= end:
            return identifier
    # 特殊的 code 处理
    if 0 <= code <= 0:
        return CodeEnum.SUCCESS.value
    print(123123)
    return CodeEnum.SUCCESS.value


def get_response_code(response):
    # 拿到 code
    if response and type(response) == dict:
        code = response.get('code') or response.get('Code') or response.get('CODE')
    else:
        code = None
    # 转换 code
    if code:
        code = identify_code(code)
    else:
        code = CodeEnum.SUCCESS.value
    return code


def get_response_msg(response):
    if response and type(response) == dict:
        msg = response.get('msg') or response.get('message') or response.get('Message')
    else:
        msg = None
    return msg


async def get_access_log_str(request, response, start_time, end_time, code, msg, log_type=None, **access_kwargs):
    try:
        t1 = time.time()
        method = get_request_method(request)
        process_time = get_process_time(start_time, end_time)
        # user_id = await get_user_id(request)
        request_ip = get_ip(request)
        request_header = get_header(request)
        request_data = {"ip": request_ip, "request_header": request_header}
        # TODO
        if response:
            response_body = [chunk async for chunk in response.body_iterator]
            reData = {(b''.join(response_body)).decode()}
            response.body_iterator = iterate_in_threadpool(iter(response_body))
            status = get_status(response)
        else:
            status = 500
            reData = None
        reqParams = await get_request_params(request)
        t2 = time.time()
        logRT = get_process_time(t1, t2)
        msg = str(msg).replace('\n', ' ')
        debug_msg = f"""{method}\t {status}\t {code}\t {process_time}\t {logRT}\t - \t - \t {request_data} \t {reqParams} \t {reData}\t {msg}"""
        info_msg = f"""{method}\t {status}\t {code}\t {process_time}\t {logRT}\t - \t"""
        print("info_msg", info_msg)
        access_logger.debug(str(debug_msg))
        access_logger.info(str(info_msg))
    except Exception as e:
        error_logger.error('access log error :{}'.format(str(e)))


async def get_access_error_log(request, error, msg):
    try:
        t1 = time.time()
        method = get_request_method(request)
        # user_id = await get_user_id(request)
        request_ip = get_ip(request)
        request_header = get_header(request)
        reqParams = "await get_request_params(request)"
        t2 = time.time()
        logRT = get_process_time(t1, t2)
        msg = str(msg).replace("\n", "")
        info_msg = f'{method} \t {logRT} \t - \t - \t  ip-{request_ip}, header-{request_header} \t {reqParams} \t {msg} \t {error}'
        error_logger.error(str(info_msg))
    except Exception as e:
        error_logger.error('access log error :{}'.format(str(e)))
        raise e


def get_service_log_str(func, response, start_time, end_time, code, msg, *args, **kwargs):
    try:
        t1 = time.time()
        method = func.__name__
        process_time = get_process_time(start_time, end_time)
        result = response
        t2 = time.time()
        logRT = get_process_time(t1, t2)
        debug_msg = f'{method} \t 200 \t {code}\t {msg} \t {process_time} \t {logRT} \t - \t {args}{kwargs} \t {result}'
        server_logger.info(str(debug_msg))
    except Exception as e:
        error_logger.error('service log error :{}'.format(str(e)))


"""
"time":请求时间
,"traceId":全链路Id
,"method":访问方法
,"status":http状态码
,"code":业务状态码
,"msg": 返回描述，当异常时，可以把简略堆栈放里面
,"resRT": 响应时长
,"logRT": 日志打印耗时
,"guid": 留空
,"requestId":用户访问唯一码
,"userId": 用户id
,"extObj": 扩展信息：包含用Ip,和请求header
,"reqParams": 请求参数
,"reData": 返回参数
"""
from fastapi import (
    FastAPI,
    Request,
    Response
)


def register_middleware(app: FastAPI):
    @app.middleware("http")
    async def log(request: Request, call_next):
        """
        日志记录
        :param request:
        :param call_next:
        :return:
        """
        client_ip = request.client.host
        start_time = time.time()
        try:
            response: Response = await call_next(request)
            process_time = time.time() - start_time
            msg = get_response_msg(response)
            await get_access_log_str(request, response, start_time, process_time, response.status_code, msg)
        except Exception as e:
            process_time = time.time() - start_time
            msg = str(e)
            response = None
            await get_access_log_str(request, response, start_time, process_time, 500, msg)
            raise e
        return response

    @app.middleware("http")
    async def log_middleware(request: Request, call_next):
        local_trace.trace_id = str(uuid.uuid4().hex)
        local_trace.request = request
        response = await call_next(request)
        return response


# 记录api执行日志
def access_log(**access_kwargs):
    def decorator(func):
        @functools.wraps(func)
        async def wrapper(*args, **kwargs):
            print_res: bool = access_kwargs.get('print_res') if access_kwargs.get('print_res') else False
            print_req: bool = access_kwargs.get('print_req') if access_kwargs.get('print_req') else False
            print('是否打印请求', print_res)
            print('是否打印响应', print_req)
            start_time = time.time()
            try:
                response = func(*args, **kwargs)
                # 获取code
                code = get_response_code(response)
                msg = get_response_msg(response)
            except Exception as e:
                code = None
                msg = traceback.format_exc()
                end_time = time.time()
                request = local_trace.request
                response = 'error'
                await get_access_log_str(request, response, start_time, end_time, code, msg, log_type='error',
                                         **access_kwargs)
                print("error", e)
                raise e
            end_time = time.time()
            request = local_trace.request
            await get_access_log_str(request, response, start_time, end_time, code, msg, **access_kwargs)
            return response

        return wrapper

    return decorator


"""
"time":请求时间
,"traceId":全链路Id
,"method":访问方法
,"status":http状态码
,"code":业务状态码
,"msg": 返回描述，当异常时，可以把简略堆栈放里面
,"resRT": 响应时长
,"logRT": 日志打印耗时
,"reqParams": 请求参数
,"reData": 返回参数
"""


# 记录方法执行日志
def service_log(**access_kwargs):
    def decorator(func):
        @functools.wraps(func)
        def wrapper(*args, **kwargs):
            code = None
            msg = None
            response = {}
            start_time = time.time()
            try:
                response = func(*args, **kwargs)
                if isinstance(response, dict):
                    code = str(response.get('code'))
                    msg = response.get('msg') or response.get('message')
            except:
                msg = traceback.format_exc()
            end_time = time.time()
            get_service_log_str(func, response, start_time, end_time, code, msg, *args, **kwargs)
            return response

        return wrapper

    return decorator


def class_decorator(func):
    @functools.wraps(func)
    def wrapper(*args, **kwargs):
        code = None
        msg = None
        response = {}
        start_time = time.time()
        try:
            response = func(*args, **kwargs)
            if isinstance(response, dict):
                code = get_response_code(response)
                msg = get_response_msg(response)
        except:
            msg = traceback.format_exc()
        end_time = time.time()
        get_service_log_str(func, response, start_time, end_time, code, msg, *args, **kwargs)
        return response

    return wrapper


def class_log(cls):
    for name, method in vars(cls).items():
        if callable(method):
            setattr(cls, name, class_decorator(method))
    return cls



import logging
import uuid
import os
from datetime import datetime
import sys
import threading

# 自定义trace_filter属性名
TRACE_FILTER_ATTR = "trace_filter"
# 当前线程的local_trace, 需要添加全局trace_id, 使用示例：trace.trace_id
local_trace = threading.local()
formatter = logging.Formatter(
    'time:%(asctime)s, traceId:%(trace_id)s, %(asctime)s - %(name)s - %(levelname)s - %(message)s')


class TraceFilter(logging.Filter):
    """
    通过在record中添加trace_id, 实现调用跟踪和日志打印的分离
    """

    def __init__(self, name=""):
        """
        init
        @param name: filter name
        """
        super().__init__(name)

    def filter(self, record):
        """
        重写filter方法
        @param record: record
        @return:
        """
        trace_id = local_trace.trace_id if hasattr(local_trace, 'trace_id') else uuid.uuid1()
        record.trace_id = trace_id
        return True


class TraceLogger:
    @staticmethod
    def get_log_file_path(logger_name):
        # 创建日志文件路径
        basedir = os.path.abspath('..')
        log_dir_name = 'logs'
        current_working_directory = os.getcwd()
        log_file_path = os.path.join(current_working_directory, log_dir_name)
        log_dir = os.path.join(basedir, log_file_path)
        if not os.path.exists(log_dir):
            os.mkdir(log_dir)
        log_file_name = "user_{}_log.{}.log".format(logger_name, str(datetime.now().strftime('%Y-%m-%d')))
        log_file_path = os.path.join(log_dir, log_file_name)
        return log_file_path

    @staticmethod
    def get_logger_func(logger_name):
        # 创建一个名为'access'的日志记录器实例
        access_filename = TraceLogger.get_log_file_path(logger_name)
        access_logger = logging.getLogger(logger_name)
        access_logger.setLevel(logging.INFO)  # 设置日志记录器的最低捕获级别为DEBUG
        # 添加日志跟踪filter
        trace_filter = TraceFilter()
        access_logger.addFilter(trace_filter)
        # 创建一个文件处理器，用于输出logger1的日志到'debug.log'
        access_info = logging.FileHandler(access_filename, encoding='utf-8', delay=False)
        access_info.setLevel(logging.INFO)  # 设置处理器的级别为DEBUG
        # 将格式化器添加到处理器中
        access_info.setFormatter(formatter)
        # 将处理器添加到日志记录器中
        access_logger.addHandler(access_info)
        return access_logger


# 创建一个日志格式化器


access_logger = TraceLogger.get_logger_func("access")
server_logger = TraceLogger.get_logger_func("server")
error_logger = TraceLogger.get_logger_func("error")

# error_logger = TraceLogger.get_logger()
console_handler = logging.StreamHandler()
# logger = TraceLogger.get_logger()
# server_logger = TraceLogger.get_server_logger()
# error_logger = TraceLogger.get_server_logger()



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
"""
@Author  ：LiShun
@File    ：exception.py
@Time    ：2022/7/12 18:56
@Desc    ：
"""
__all__ = [
    'BaseHTTPException', 'register_exception'
]

import traceback
from typing import Any, Optional, Dict

from fastapi import FastAPI
from fastapi import Request
from fastapi.exceptions import HTTPException, RequestValidationError
from fastapi.responses import JSONResponse, Response

# from base_code.response import json_response
from .base_code.code import CODE
from .log_wrapper import get_access_error_log


def json_response(data=None, message=None, code=200, status_code=200) -> Response:
    code_dict = CODE.get(code) or CODE[400]
    return JSONResponse(
        status_code=status_code,
        content={
            "code": code_dict["code"],
            "message": message or code_dict["zh-cn"],
            "data": data
        }
    )


class BaseHTTPException(HTTPException):
    EXC_STATUS_CODE = 500
    EXC_CODE = 1000
    EXC_MESSAGE = None

    def __init__(
            self,
            message: Any = None,
            status_code: int = 500,
            code: int = 40000,
            headers: Optional[Dict[str, Any]] = None
    ) -> None:
        self.message = message or self.EXC_MESSAGE
        self.status_code = status_code or self.EXC_STATUS_CODE
        self.code = code or self.EXC_CODE
        self.headers = headers

    def __repr__(self) -> str:
        class_name = self.__class__.__name__
        return f"{class_name}(status_code={self.status_code!r}, message={self.message!r})"


def register_exception(app: FastAPI):
    """
    捕获FastApi异常
    :param app:
    :return:
    """

    @app.exception_handler(Exception)
    async def exception_handle(request: Request, exc: Exception):
        msg = traceback.format_exc()
        await get_access_error_log(request, error=str(exc), msg=msg)
        return json_response(code=400, message=str(exc), status_code=200)

    @app.exception_handler(HTTPException)
    async def http_exception_handle(request: Request, exc: HTTPException):
        msg = traceback.format_exc()
        await get_access_error_log(request, error=str(exc), msg=msg)
        return json_response(code=500, message=str(exc.detail), status_code=exc.status_code)

    @app.exception_handler(BaseHTTPException)
    async def base_http_exception_handle(request: Request, exc: BaseHTTPException):
        msg = traceback.format_exc()
        await get_access_error_log(request, error=str(exc), msg=exc)
        return json_response(code=500, message=str(exc.message), status_code=exc.status_code)

    @app.exception_handler(AssertionError)
    async def assert_exception_handle(request: Request, exc: AssertionError):
        exc_str = ''.join(exc.args)
        msg = traceback.format_exc()
        await get_access_error_log(request, error=f"assert_exception > {exc_str}", msg=msg)
        return json_response(code=500, message=exc_str, status_code=500)

    @app.exception_handler(RequestValidationError)
    async def request_exception_handler(request: Request, exc: RequestValidationError):
        exc_str = f'{exc}'.replace('\n', ' ').replace('   ', ' ')
        msg = traceback.format_exc()
        await get_access_error_log(request, error=f"request_exception > {exc_str}", msg=msg)
        return json_response(code=500, message=exc_str, status_code=500)


# 自定义异常类
class TableNameError(Exception):
    __doc__ = "数据库表名命名规则验证错误"


class ColumnNameError(Exception):
    __doc__ = "数据库表属性命名规则验证错误"


class RepetitiveError(Exception):
    __doc__ = "数据库表字段重复错误"


class DataNotFoundError(Exception):
    __doc__ = "数据库数据未找到"


class ParamLackError(Exception):
    __doc__ = "缺少参数"


class ParamValidatedError(Exception):
    __doc__ = "自定义参数格式不正确"


class AlreadyExistsError(Exception):
    __doc__ = "资源已存在"


class CustomRaiseError(Exception):
    __doc__ = "封装自定义报错"
    """
    使用框架自动捕获异常时候使用
    use:
        raise CustomRaiseError(*e.args, int)
    params:
        *e.args: 原始的错误信息
        int: 想要的业务报错信息code
    """



__version__ = "0.1.10"

from .exception import register_exception as register_exception
from .log_wrapper import register_middleware as register_middleware
from .log_wrapper import class_log as class_log
from .log_wrapper import access_log as access_log
